package be.iminds.ilabt.jfed.experimenter_gui.bugreporting;

import be.iminds.ilabt.jfed.call_log_output.HtmlLogOutput;
import be.iminds.ilabt.jfed.call_log_output.SerializableApiCallDetails;
import be.iminds.ilabt.jfed.experiment.tasks.ExperimentTaskStatus;
import be.iminds.ilabt.jfed.log.cache.ApiCallDetailsCache;
import org.rendersnake.HtmlCanvas;

import java.io.IOException;
import java.time.Duration;
import java.time.Instant;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * User: twalcari
 * Date: 12/11/2015
 * Time: 11:29
 */
public class SerializableStitchingJobReportWriter extends HtmlLogOutput {

    private final SerializableJobReport jobReport;
    private final ApiCallDetailsCache cache;

    public SerializableStitchingJobReportWriter(SerializableJobReport jobReport, ApiCallDetailsCache cache) {
        super("Start Stitching Experiment Report");
        this.jobReport = jobReport;
        this.cache = cache;
    }

    @Override
    public void htmlContent(HtmlCanvas html) throws IOException {
        addClassVisibleToggle(html, "logline-DEBUG", "Debug Log Lines");

        int index = 0;
        for (SerializableJobStateReport reportState : jobReport.getJobStateReports()) {
            List<SerializableApiCallDetails> callsCopy;
            synchronized (reportState.getTaskExecutions()) {
                callsCopy = reportState.getTaskExecutions().stream()
                        .filter(Objects::nonNull)
                        .flatMap(taskExecution -> taskExecution.getApiCallDetails().stream())
                        .map(cache::getApiCallDetails).collect(Collectors.toList());
            }
            writeGroup(html,
                    index++,
                    convertExperimentTaskStatus(reportState.getStatus()),
                    reportState.getName() + " calls",
                    "Executed tasks: " + reportState.getTaskExecutions().stream().map(SerializableTaskExecution::getName).collect(Collectors.joining(", ")),
                    Duration.between(reportState.getStart(),
                            reportState.getEnd() != null ? reportState.getEnd() : Instant.now()).toMillis(),
                    Date.from(reportState.getStart()),
                    Date.from(reportState.getEnd()),
                    null/*exception*/,
                    callsCopy,
                    reportState.getLogEntries());
        }
    }

    private static LogEntry convertLogLine(be.iminds.ilabt.jfed.experimenter_gui.bugreporting.LogLine logLine) {
        String newlines = logLine.getMessage() == null ? "" : logLine.getMessage().replaceAll("[^\n]*", "");
        return new LogEntry(LogLineType.valueOf(logLine.getLevel()), logLine.getMessage(), logLine.getTime(), logLine.getException(), newlines.length() > 3/*showAsCode*/);
    }

    private static TestResultState convertExperimentTaskStatus(ExperimentTaskStatus status) {
        switch (status) {
            case SUCCESS:
                return TestResultState.SUCCESS;
            case FAILED:
                return TestResultState.FAILED;
            case WARNING:
                return TestResultState.WARN;
            default:
                return null;

        }
    }
}
